/***************************************************************************
                          cddb.cpp  -  CDDB and cdparanoia classes
                             -------------------
    begin                : Sat Aug 18 20:20:00 /etc/localtime 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cddb.h"

/**
 * Constructor
 * @param QString Path to cdparanoia if it is not in path
 * @param CDDBtitleInfo Pointer To Struct
 */
cdparanoia::cdparanoia(const QString &fpath, struct CDDBtitleInfo &info)
{
	cerr << "cdparanoia::initialized" << endl;
	path = validateFilename(fpath);
	titleInfo= info;
	// connect signals for processes. This should be cleaner since it happens only once.
	QApplication::connect(&p, SIGNAL(receivedStderr(KProcess *, char *, int)), this, SLOT(procOutput(KProcess *, char *, int)));
	QApplication::connect(&p, SIGNAL(processExited(KProcess *)), this, SLOT(procExit(KProcess *)));

	// establish time for timeout
	cdptimer = new QTimer(this);
	if (!cdptimer)
		exit(1);
	connect(cdptimer, SIGNAL(timeout()), this, SLOT(cdpTimeout()));
}

/**
 * Destructor
 */
cdparanoia::~cdparanoia()
{
	delete cdptimer;
}

/** Called when the cdparanoia timer times out. This function kills the current process, emits error() and returns. */
void
cdparanoia::cdpTimeout()
{
	cerr << "cdparanoia::timeout" << endl;
	if (p.isRunning())
		p.kill();
	cdptimer->stop();
	emit error(PROCESS_TIMEOUT);

}

/** Checks to find out where cdparanoia is located.
 */
bool cdparanoia::checkPath()
{
	return checkFileExists(path, "cdparanoia");
}

/** Processes the output of cdparanoia after it is done executing.
 * @param KProcess Process
 */
void
cdparanoia::procExit(KProcess *proc)
{
	// everything is cool, turn off timer.
	cdptimer->stop();

	if (!proc->normalExit())
	{
		cerr << "cdparanoia::process did not exit normally" << endl;
		emit error(PROCESS_ERROR);
		goto finish;
	}
	cerr << "cdparanoia::process exited normally" << endl;
	kapp->processEvents(3000);
	if (proc->exitStatus() > 0)
	{
		cerr << "cdparanoia::no audio cd found" << endl;
		cdparaout = "";
		emit error(CDPARA_NO_CD);
		return;
	}
	else
	{
		cerr << "cdparanoia::audio cd found" << endl;
		int a=0;
		if ((a=processEvents()) < 0)
			emit error(a);
	}
finish:
	emit cdpFinish();
}

/** Just appends the buffer to cdparaout.
 * @param KProcess The Process
 * @param char Pointer to the data returned
 * @param int Amount of data available
 */
void
cdparanoia::procOutput(KProcess *, char *buff, int)
{
	cdparaout.append(buff);
}

/** Get cdparanoia information from CD-ROM drive.
 */
void
cdparanoia::get()
{
	emit updateGUI(5, i18n("Checking for CD..."));

	cdparaout = "";
	p.kill();
	p.clearArguments();
	if (checkPath())
		p << path+"/cdparanoia";
		//p.setExecutable(path + "/cdparanoia");
	else
		p << "cdparanoia";
		//p.setExecutable("cdparanoia");
	p << "-d" << validateFilename(titleInfo.device) << "-Q";
	// start the execution and capture the output and check for error message.
	kapp->processEvents();
	bool procstatus = p.start(KProcess::NotifyOnExit, KProcess::Stderr);
	if (!procstatus)
	{
		emit error(CDPARA_NOT_FOUND);
		emit cdpFinish();
		return;
	}
	cdptimer->start(DEFAULT_SOCKET_TIMEOUT * 1000, TRUE);
}

/**
 * Process the output of cdparanoia
 */
int cdparanoia::processEvents()
{
	// init variables
	QValueList<int>::Iterator it;
	long location = 0, begin = 0, end = 0, totallocation = 0;
	QString temp;
	int cdtotal, residual;
	cerr << "cdparanoia::processEvents:\n" << cdparaout << endl;
	cdparaout=cdparaout.simplifyWhiteSpace();

	//----------------- program header output checking ----------------------
	emit updateGUI(10, i18n("Retrieving CD data..."));
	kapp->processEvents();

	location = cdparaout.find("cdparanoia III");
	if (location == -1)
		return CDPARA_WRONG_VERSION;

	location = cdparaout.find("Table of contents", location);
	if (!location)
		return CDPARA_BAD_DATA;

	titleInfo.trackSectorBegin.clear();
	titleInfo.trackSectorLength.clear();
	titleInfo.trackData.clear();
	titleInfo.cddb_genre = "";
	titleInfo.disc_id = "";
	titleInfo.title.name = "";
	titleInfo.totalLength = "";

	//------------------------- process table -----------------------
	// find total location
	totallocation = cdparaout.find("TOTAL", location);

	do
	{
		// set start location to first track
		location = cdparaout.find(".", location);

		if (totallocation < location)
			break;

		// find first digit
		location+=2;
		begin = location;
		//find last digit
		location=cdparaout.find("[",location);
		end=location-2;
		//convert to int
		temp = cdparaout.mid(begin,(end - begin + 1));
		titleInfo.trackSectorLength.append(temp.toInt());
		//find first digit again
		location = cdparaout.find("]", location);
		begin=location+2;
		//find last digit again
		location = cdparaout.find("[", location);
		end = location-2;
		// convert to int
		temp = cdparaout.mid(begin,(end - begin + 1));
		titleInfo.trackSectorBegin.append(temp.toInt()+150);
		//seek last period in line
		location = cdparaout.find(".", location);
		location++;
	} while (location != -1);

	//--------------------------- get cd total ----------------------
	emit updateGUI(14, i18n("Retrieving CD data..."));
	for (it = titleInfo.trackSectorBegin.begin();it != titleInfo.trackSectorBegin.end(); ++it)
		cerr << "cdparanoia::processEvents:begin: " << (*it) << endl;
	//find first digit of total
	begin = totallocation+6;
	//find last digit again
	end=cdparaout.find("[",begin)-2;
	// convert to int
	temp = cdparaout.mid(begin,(end - begin + 1));
	titleInfo.totalLength = QString::number(temp.toInt()+150);
	cdtotal =titleInfo.totalLength.toInt();
	// calculate second variable for cdid calculation
//	if (((cdtotal-titleInfo.trackSectorBegin[0]) % 75) > 37)
//		cdtotal+=150;
//	else
//		cdtotal+=225;
	cdtotal=(cdtotal)/75;
	cerr << "cdparanoia::processEvents:totalLength: " << titleInfo.totalLength << " " << cdtotal << endl;

	//-------------------------- number crunching ----------------------
	emit updateGUI(16, i18n("Retrieving CD data..."));
	// calculate residual(first variable in disc id calculation)
	residual = cddbSum();
	cerr << "cdparanoia::processEvents:residual: " << residual << endl;

	// calculate disc_id
	unsigned long int disc_id_number =(residual % 0xff) << 24 | (cdtotal-2) << 8 | titleInfo.trackSectorBegin.count();
	titleInfo.disc_id.sprintf("%08lx", disc_id_number);
	cerr << "cdparanoia::processEvents:disc_id: " << titleInfo.disc_id << endl;

	//---------------------- hours, minutes, seconds --------------------------------
	emit updateGUI(20, i18n("Retrieving CD data..."));
	// calculate track lengths in hh:mm:ss
	int hours = 0, minutes = 0, seconds = 0, totalseconds = 0;
	trackInfo dud;
	for (it = titleInfo.trackSectorLength.begin();it != titleInfo.trackSectorLength.end(); ++it)
	{
		//75 sectors per second.
		totalseconds =(*it) / 75;
		//We must round up if there is over half a sector worth of sound at the end
		if ((*it % 75) > 37)
			totalseconds++;

		if (totalseconds >= 3600)
		{
			hours =totalseconds / 3600;
			totalseconds -=(hours * 3600);
		}
		else
			hours=0;
		if (totalseconds >= 60)
		{
			minutes =totalseconds / 60;
			totalseconds -= minutes * 60;
		}
		else
			minutes=0;
		seconds = totalseconds;

		if (hours > 23)
		{
			hours = 23;
			minutes=59;
			seconds=59;
		}

		dud.length = returnLength(hours, minutes, seconds);
		titleInfo.trackData.append(dud);

	}
	emit cdpDone(titleInfo);
	return EVERYTHING_OK;
}
/**
 * Does part of the summation for calculating disc_id.
 *
 * Returns sum.
 */
int cdparanoia::cddbSum()
{
	QValueList < int >::Iterator it;
	int s, residual=0;
	for (it = titleInfo.trackSectorBegin.begin();it != titleInfo.trackSectorBegin.end(); ++it)
	{
		 s=(*it)/75;
		 /* For backward compatibility this algorithm must not change */
		 while (s > 0) {
			residual = residual +(s % 10);
			s = s / 10;
		 }
	}
	return residual;
}










/**
 * Constructor
 * @param QWidget Parent Widget
 * @param CDDBtitleInfo CDDBtitleInfo to set default info and pass track information
 * @param CDDBserverInfo CDDBserverInfo holds server/protocol information
 */
cddb::cddb(QWidget *parent, struct CDDBtitleInfo finfo, struct CDDBserverInfo *si) : QObject(parent)
{
	cerr << "cddb::initialized" << endl;
	// save the data
	titleInfo=finfo;

	if (si)
	{
		serverInfo.get_protocol=si->get_protocol;
		serverInfo.put_protocol=si->put_protocol;
		serverInfo.cddbp_server=si->cddbp_server;
		serverInfo.cddbp_port=si->cddbp_port;
		serverInfo.http_server=si->http_server;
		serverInfo.http_port=si->http_port;
		serverInfo.http_cgi=si->http_cgi;
		serverInfo.proxy_server=si->proxy_server;
		serverInfo.proxy_port=si->proxy_port;
		serverInfo.smtp_server=si->smtp_server;
		serverInfo.smtp_port=si->smtp_port;
		serverInfo.kde_email_profile=si->kde_email_profile;
		serverInfo.check_cddb_local=si->check_cddb_local;
		serverInfo.local_cddb_path=si->local_cddb_path;
		serverInfo.cddb_email=si->cddb_email;
		strncpy(serverInfo.host,si->host,255);
	}

	state=CDDB_STATE_END; //safe value

	// since we dont use qt or kde sockets, we manually emit signals on an as needed basis(with exception of qtimer)
	connect(this, SIGNAL(sigconnected()), this, SLOT(cddb_spincycle()));
}

/**
 * Destructor
 */
cddb::~cddb()
{
	disconnect(this, SIGNAL(sigconnected()), this, SLOT(cddb_spincycle()));
}

/** get function for retrieving CDDB info. Must have cdparanoia information first.
 */
void
cddb::get()
{
	cerr << "cddb::get()" << endl;

	// If local query is enabled, we try that first. If not make a tcp/ip connection.
	if (serverInfo.check_cddb_local==TRUE)
		state=CDDB_STATE_CDDBLOCAL;
	else
		state=CDDB_STATE_CONNECTING;

	// if tcp/ip connection is made, we need to set the host/port.
	// also the next state after connection must be set.
	if (serverInfo.get_protocol==CDDB_PROTO_CDDBP)
	{
		chost=serverInfo.cddbp_server;
		cport=serverInfo.cddbp_port;
		preservestate=CDDB_STATE_CDDBHELLO;
	}
	else if (serverInfo.get_protocol==CDDB_PROTO_HTTP)
	{
		chost=serverInfo.http_server;
		cport=serverInfo.http_port;
		preservestate=CDDB_STATE_CDDBQUERY;
	}
	else if (serverInfo.get_protocol==CDDB_PROTO_PROXY)
	{
		chost=serverInfo.proxy_server;
		cport=serverInfo.proxy_port;
		preservestate=CDDB_STATE_CDDBQUERY;
	}
	else
		state=CDDB_STATE_END;

	emit updateGUI(21, i18n("Connecting to CDDB..."));
	//connect to host
	cddb_spincycle();
}

/** For submitting to CDDB.
 * @param QString Artist Name
 */
void
cddb::put(const QString &f_artist)
{
	cerr << "cddb::put()" << endl;
	state=CDDB_STATE_CONNECTING;
	preservestate=CDDB_STATE_SMTPHELLO;

	if (serverInfo.cddb_email.isEmpty() || serverInfo.kde_email_profile.isEmpty() || f_artist.isEmpty() )
	{
		emit error(SMTP_MAIL_FAIL);
		state=CDDB_STATE_END;
	}
	// set ports
	chost=serverInfo.smtp_server;
	cport=serverInfo.smtp_port;
	// set titlinfo to local title info
	artist = f_artist;

	emit updateGUI(21, i18n("Connecting to Email Server..."));
	// connect directly to net and set the next state after connection
	cddb_spincycle();
}
/** For searching CDDB.
 * @param QString Search String
 * @param bool Artist Enabled Or Disabled
 * @param bool Title Enabled Or Disabled
 */
void cddb::search(const QString &ssearch, const bool &aartist, const bool &ttitle)
{
	cerr << "cddb::search()" << endl;
	state=CDDB_STATE_CONNECTING;
	preservestate=CDDB_STATE_SEARCHQUERY;

	if (ssearch.isEmpty())
	{
		emit error(CDDB_NO_MATCH);
		state=CDDB_STATE_END;
	}

	s_text=ssearch;
	s_artist=aartist;
	s_title=ttitle;

	emit updateGUI(21, i18n("Connecting to CDDB..."));
	//connect to host
	serverInfo.http_server="www.freedb.org";
	serverInfo.http_port="80";
	if ((serverInfo.get_protocol==CDDB_PROTO_CDDBP) ||(serverInfo.get_protocol==CDDB_PROTO_HTTP))
		serverInfo.get_protocol=CDDB_PROTO_SEARCH;
	else if (serverInfo.get_protocol==CDDB_PROTO_PROXY)
		serverInfo.get_protocol=CDDB_PROTO_SEARCHPROXY;
	else
		state=CDDB_STATE_END;

	if (serverInfo.get_protocol==CDDB_PROTO_SEARCH)
	{
		chost=serverInfo.http_server;
		cport=serverInfo.http_port;
	}
	else if (serverInfo.get_protocol==CDDB_PROTO_SEARCHPROXY)
	{
		chost=serverInfo.proxy_server;
		cport=serverInfo.proxy_port;
	}
	else
		state=CDDB_STATE_END;
	cddb_spincycle();
}
/** For retrieving titles if you already know the genre and discid.
 * @param QString CDDB Category
 * @param QString CDDB Disc ID
 */
void cddb::simpleGet(const QString &cat, const QString &discid)
{
	cerr << "cddb::simpleGet()" << endl;
	// If local query is enabled, we try that first. If not make a tcp/ip connection.
	preservestate=CDDB_STATE_SEARCHREAD;
	if (serverInfo.check_cddb_local)
		state=CDDB_STATE_CDDBLOCAL;
	else
		state=CDDB_STATE_CONNECTING;

	if (cat.isEmpty() || discid.isEmpty())
	{
		emit error(CDDB_NO_MATCH);
		state=CDDB_STATE_END;
	}

	// if tcp/ip connection is made, we need to set the host/port.
	// also the next state after connection must be set.
	if ((serverInfo.get_protocol==CDDB_PROTO_CDDBP) || (serverInfo.get_protocol==CDDB_PROTO_SEARCH))
		serverInfo.get_protocol=CDDB_PROTO_HTTP;
	else if (serverInfo.get_protocol==CDDB_PROTO_SEARCHPROXY)
		serverInfo.get_protocol=CDDB_PROTO_PROXY;

	if (serverInfo.get_protocol==CDDB_PROTO_HTTP)
	{
		chost=serverInfo.http_server;
		cport=serverInfo.http_port;
	}
	else if (serverInfo.get_protocol==CDDB_PROTO_PROXY)
	{
		chost=serverInfo.proxy_server;
		cport=serverInfo.proxy_port;
	}
	else
		state=CDDB_STATE_END;

	titleInfo.disc_id=discid;
	titleInfo.cddb_genre=cat;

	emit updateGUI(21, i18n("Connecting to CDDB..."));
	//connect to host
	cddb_spincycle();
}

/** Retrieves the CD information from CDDB. Main wrapper function.
 */
void
cddb::cddb_spincycle()
{
	int a = 0;

	do
	{
		switch(state)
		{
			case CDDB_STATE_CDDBLOCAL: //will run through and check for local. if match exit, if not check for protocol and set state
				kapp->processEvents();
				emit updateGUI(30, i18n("Local CDDB Lookup..."));
				cerr << "cddb::spincycle:CDDB_STATE_CDDBLOCAL" << endl;
				if (serverInfo.get_protocol != CDDB_PROTO_EMAIL)
				{
					a=queryLocalCDDB();
					if (a==CDDB_NO_MATCH)
						state=CDDB_STATE_CONNECTING;
					else if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_CONNECTING;
					}
					else
					{
						emit titleReceived(titleInfo);
						state=CDDB_STATE_END;
					}
				}
				else
					state=CDDB_STATE_END;
				break;
			case CDDB_STATE_CONNECTING:
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_CONNECTING" << endl;
				state=preservestate; //we have to set the state before connect, otherwise it will recurse.
				if (connectToServer(chost,cport) < 0)
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_CDDBHELLO: //make sure proto is cddbp, then send a cddb hello
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_CDDBHELLO" << endl;
				if (serverInfo.get_protocol == CDDB_PROTO_CDDBP)
				{
					emit updateGUI(40, i18n("Saying hello to CDDB..."));
					a = helloCDDB();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					state=CDDB_STATE_CDDBQUERY;
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_CDDBQUERY: //query the system. if proto !=cddbp then shutdown. make sure proto not email.
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_CDDBQUERY" << endl;
				if (serverInfo.get_protocol != CDDB_PROTO_EMAIL )
				{
					emit updateGUI(41, i18n("Querying for CD..."));
					a = queryForCD();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					if (serverInfo.get_protocol == CDDB_PROTO_CDDBP)
						state=CDDB_STATE_CDDBREAD;
					else
					{
						//shutdown, set preserve, set state
						shutdown(sockfd, SHUT_RDWR);
						preservestate=CDDB_STATE_CDDBREAD;
						state=CDDB_STATE_CONNECTING;
					}
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_CDDBREAD: // read cd from cddb. make sure proto not email. if proto != cddbp then shutdown.
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_CDDBREAD" << endl;
				if (serverInfo.get_protocol != CDDB_PROTO_EMAIL )
				{
					emit updateGUI(45, i18n("Retrieving CD Info..."));
					a = retrieveCDInfo();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					state=CDDB_STATE_CDDBQUIT;
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SMTPHELLO: // make sure proto email. send email helo
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_SMTPHELLO" << endl;
				if (serverInfo.put_protocol == CDDB_PROTO_EMAIL )
				{
					emit updateGUI(40, i18n("Saying hello to SMTP..."));
					a = helloSMTP();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					state=CDDB_STATE_SMTPSENDHEADER;
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SMTPSENDHEADER: // make sure proto email. send header.
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_SMTPSENDHEADER" << endl;
				if (serverInfo.put_protocol == CDDB_PROTO_EMAIL )
				{
					emit updateGUI(50, i18n("Sending SMTP Header..."));
					a = sendMailHeader();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					state=CDDB_STATE_SMTPSENDBODY;
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SMTPSENDBODY: // make sure proto email. send body.
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_SMTPSENDBODY" << endl;
				if (serverInfo.put_protocol == CDDB_PROTO_EMAIL )
				{
					// send body
					emit updateGUI(60, i18n("Sending SMTP Body..."));
					a = sendMailBody();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					state=CDDB_STATE_SMTPQUIT;
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_CDDBQUIT: //for get_protocol only;
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_CDDBQUIT" << endl;
				if (serverInfo.get_protocol == CDDB_PROTO_CDDBP)
				{
					emit updateGUI(55, i18n("Saying goodbye to CDDB..."));
					a = goodByeCDDB();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					emit titleReceived(titleInfo);
				}
				else if ((serverInfo.get_protocol == CDDB_PROTO_HTTP) ||(serverInfo.get_protocol == CDDB_PROTO_PROXY) )
					emit titleReceived(titleInfo);
				state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SMTPQUIT: // for put_protocol only
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_SMTPQUIT" << endl;
				if (serverInfo.put_protocol == CDDB_PROTO_EMAIL)
				{
					// say goodbye and inform user
					emit updateGUI(70, i18n("Saying goodbye to SMTP..."));
					a = goodByeSMTP();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					emit submitSuccess(titleInfo);
				}
				state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SEARCHQUERY:
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_SEARCHQUERY" << endl;
				emit updateGUI(30, i18n("Searching CDDB..."));
				a=searchQuery();
				if (a < 0)
				{
					emit error(a);
					state=CDDB_STATE_SHUTDOWN;
					break;
				}
				state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SEARCHREAD: // read cd from cddb. make sure proto not email. if proto != cddbp then shutdown.
				kapp->processEvents();
				cerr << "cddb::spincycle:CDDB_STATE_CDDBREAD" << endl;
				if (serverInfo.get_protocol != CDDB_PROTO_EMAIL )
				{
					emit updateGUI(45, i18n("Retrieving CD Info..."));
					a = retrieveSearchInfo();
					if (a < 0)
					{
						emit error(a);
						state=CDDB_STATE_SHUTDOWN;
						break;
					}
					state=CDDB_STATE_CDDBQUIT;
				}
				else
					state=CDDB_STATE_SHUTDOWN;
				break;
			case CDDB_STATE_SHUTDOWN: //send a shutdown, then end
				cerr << "cddb::spincycle:CDDB_STATE_SHUTDOWN" << endl;
				kapp->processEvents();
				shutdown(sockfd, SHUT_RDWR);
				state=CDDB_STATE_END;
				break;
			case CDDB_STATE_END:
				cerr << "cddb::spincycle:CDDB_STATE_END" << endl;
				emit cddbFinished();
				// this is the key to the do/while loop to exit for good
				state=99999;
				break;
			default:
				state=CDDB_STATE_END;
		};
	} while (state != 99999); //this runs until I set the state to END. END means normal exit.
}

/** Says hello to CDDB.
 */
int
cddb::helloCDDB()
{
	QString output;
	output = "cddb hello ";
	output.append(getenv("USER"));
	output.append(" ");
	output.append(serverInfo.host);
	output.append(" kmusicdb ");
	output.append(VERSION);
	output.append("\n");

	if (write_proto(output, serverInfo.get_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	QString receive;
	int server_return=read_proto(receive, serverInfo.get_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return > 201)
		return CDDB_SERVER_BAD_RESPONSE;

	server_return=read_proto(receive, serverInfo.get_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	if (server_return > 200)
		return CDDB_SERVER_BAD_RESPONSE;

	return EVERYTHING_OK;
}

/** Says goodbye to CDDB.
 */
int
cddb::goodByeCDDB()
{
	QString output;
	output = "quit\n";

	if (write_proto(output, serverInfo.get_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	return EVERYTHING_OK;

}

/** Sends the query to CDDB.
 */
int
cddb::queryForCD()
{
	// assemble query
	QString output, receive;
	QValueList < int >::Iterator it;
	int location1 = 0, location2 = 0;

	output = "cddb query ";
	output.append(titleInfo.disc_id);
	output.append(" ");
	output.append(QString::number(titleInfo.trackSectorBegin.count()));

	for (it = titleInfo.trackSectorBegin.begin(); it != titleInfo.trackSectorBegin.end(); ++it)
	{
		output.append(" ");
		output.append(QString::number(*it));
	}
	output.append(" ");
	output.append(QString::number(titleInfo.totalLength.toInt()/75));
	output.append('\n');

	if (write_proto(output, serverInfo.get_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	int server_return=read_proto(receive, serverInfo.get_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;

	// handle one, many, no matches or error
	// on a single match, the server already returned the information we need.
	// the other codes we need to do something different.
	// save the title name, discid and genre for next function
	switch(server_return)
	{
		case 200:			// 200 and 201 are normal single matches. we need do nothing
		case 201:
			cerr << "cddb::queryForCD:200/201:" << endl;
			location2 = receive.find(" ", location1 + 1);
			location1 = location2;
			break;
		case 202:			// 202 cd not found.
			cerr << "cddb::queryForCD:202" << endl;
			return CDDB_NO_MATCH;
		case 210:			// multiple exact/inexact matches found. we need an extra read.
		case 211:
			// do extra read, then we have the info we need
			cerr << "cddb::queryForCD:210/211: " << endl;
			receive = "";
			if (read_proto(receive, CDDB_PROTO_CDDBP) < 0)
				return SOCKET_READ_FAIL;
			break;
		default:
			return CDDB_UNKNOWN_ERROR;
	};

	// process results
	location2 = receive.find(" ", location1 + 1);
	titleInfo.cddb_genre = receive.mid(location1, location2 - location1);
	location1 = location2;
	location2 = receive.find(" ", location1 + 1);
	titleInfo.disc_id = receive.mid(location1, location2 - location1);
	location1 = location2;
	location2 = receive.find("/", location1 + 3) + 2;
	if (location2 == 1)
	{
		cerr << "cddb::queryForCD:bad format" << endl;
		location2 = location1 + 1;
	}
	titleInfo.title.name = receive.mid(location2);
	titleInfo.title.name.truncate(titleInfo.title.name.length()-2);
	cerr << "cddb::queryForCD: genre: " << titleInfo.
		cddb_genre << "\tcdid: " << titleInfo.disc_id << "\tname: " << titleInfo.
		title.name << endl;

	return EVERYTHING_OK;
}

/** The work horse, retrieves and processes the title information.
 */
int
cddb::retrieveCDInfo()
{
	// format string
	QString output, receive="", temp="";
	int location1 = 0, location2 = 0;
	int trackNum=0, prevNum=-1;

	output = "cddb read ";
	output.append(titleInfo.cddb_genre);
	output.append(" ");
	output.append(titleInfo.disc_id);
	output.append("\n");

	// write to socket
	if (write_proto(output, serverInfo.get_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	int server_return=read_proto(temp, serverInfo.get_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;


	// read data, check for errors, parse output and save
	do
	{
		temp = "";
		// calculate length to keep the find() in while () fast.
		if (read_proto(temp, CDDB_PROTO_CDDBP) < 0)
			return SOCKET_READ_FAIL;

		if (!temp.find("EXTD="))
			break;
		receive.append(temp);
	}
	while (temp.length());

	// find title name
	location1 = receive.find("DTITLE");
	location1 = receive.find("/", location1);
	location1 += 2;
	location2 = receive.find("\n", location1);
	titleInfo.title.name = receive.mid(location1, location2 - location1);
	cerr << "cddb::retrieveCDInfo:Title Name: " << titleInfo.title.name << endl;

	//truncate everything BEFORE first TTITLE
	receive.remove(0, receive.find("TTITLE"));
	location1=0;
	location2=0;
	do
	{
		location1 = receive.find("TTITLE", location2);
		if (location1 != -1)
		{
			// get the track number
			location1+=6;
			location2 = receive.find("=", location1);
			trackNum=receive.mid(location1, location2-location1).toInt();
			// now get the name of the track
			location1 = location2+1;
			location2 = receive.find("\n", location1);
			// some very long tracks wrap to the next line
			// so if the new tracknum matches the last track we append
			// otherwise it's a new track and we make a new entry
			if (prevNum==trackNum)
			{
				titleInfo.trackData[trackNum].name.append(receive.mid(location1, location2-location1));
			}
			else
			{
				// but it also provides a buffer on error(although I haven't seen any here)
				titleInfo.trackData[trackNum].name = receive.mid(location1, location2 - location1);
				prevNum=trackNum;
			}
			// output result, strip '\n', truncate to 60(the db track size)
			cerr << "cddb::retrieveCDInfo:TrackNum: " << trackNum << " Length: " << titleInfo.trackData[trackNum].name.length();
			cerr << " Name: " << titleInfo.trackData[trackNum].name << endl;
			titleInfo.trackData[trackNum].name.truncate(60);
			if (titleInfo.trackData[trackNum].name[titleInfo.trackData[trackNum].name.length()-1] == '\n')
				titleInfo.trackData[trackNum].name.remove(titleInfo.trackData[trackNum].name.length() - 1, 1);
		}
	} while (location1 != -1);

	return EVERYTHING_OK;
}


/** The work horse, retrieves and processes the title information.
 */
int
cddb::retrieveSearchInfo()
{
	QString output, receive="", temp;
	int location1=0, location2=0, hours=0, minutes=0, seconds=0, totalseconds=0;
	int trackNum=0, prevNum=0;
	int marker=0;
	trackInfo dud;
	QValueList<int>::Iterator it;

	// format string
	output = "cddb read ";
	output.append(titleInfo.cddb_genre);
	output.append(" ");
	output.append(titleInfo.disc_id);
	output.append("\n");

	// write to socket
	if (write_proto(output, serverInfo.get_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	int server_return=read_proto(temp, serverInfo.get_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;


	// read data, check for errors, parse output and save
	do
	{
		temp.truncate(0);
		// calculate length to keep the find() in while () fast.
		if (read_proto(temp, CDDB_PROTO_CDDBP) < 0)
			return SOCKET_READ_FAIL;

		if (temp.find("EXTD=") == 0)
			break;
		receive.append(temp);
	}
	while (temp.length());

	// Read track offsets, store length of tracks. Also required to add tracks here
	location1 = receive.find("Track frame offsets");
	location1 = receive.find("#",location1)+1;
	if (receive[location1]!='\t')
	{
		cerr << "cddb:retrieveSearchInfo:Space Format" << endl;
		marker=8;
		location1+=5;
	}
	else
	{
		cerr << "cddb:retrieveSearchInfo:Tab Format" << endl;
		marker=4;
		location1++;
	}
	do {
		location2 = receive.find("\r", location1);
		titleInfo.trackSectorBegin.append(receive.mid(location1, location2-location1).toInt());
		location1=location2+marker;
	} while (receive.mid(location1, location2-location1).find("length") != -1);
	location1=receive.find("length:",0)+8;
	location2=receive.find("\r",location1)-8;
	titleInfo.trackSectorBegin[titleInfo.trackSectorBegin.count()-1]=receive.mid(location1, location2-location1).toInt()*75;
	for (uint i=0; i<titleInfo.trackSectorBegin.count()-1;i++)
	{
		cerr << "cddb:retrieveSearchInfo:SECTOR i+1 BEGIN: " << titleInfo.trackSectorBegin[i+1] << " SECTOR i BEGIN: " << titleInfo.trackSectorBegin[i] << endl;
		titleInfo.trackSectorLength.append(titleInfo.trackSectorBegin[i+1]-titleInfo.trackSectorBegin[i]);
	}

	// Read title
	location1 = receive.find("DTITLE");
	location1 = receive.find("/", location1);
	location1 += 2;
	location2 = receive.find("\n", location1);
	titleInfo.title.name = receive.mid(location1, location2 - location1);
	cerr << "cddb::retrieveSearchInfo:Title Name: " << titleInfo.title.name << endl;

	// Read track names
	//truncate everything BEFORE first TTITLE
	receive.remove(0, receive.find("TTITLE"));
	location1=0;
	location2=0;
	for (it=titleInfo.trackSectorLength.begin();it!=titleInfo.trackSectorLength.end();it++)
	{
		//75 sectors per second.
		totalseconds =(*it) / 75;
		cerr << "cddb:retrieveSearcInfo:SECTORS: " << *it << " TOTALSECONDS: " << totalseconds;
		//We must round up if there is over half a sector worth of sound at the end
		if ((*it % 75) > 37)
			totalseconds++;
		cerr << " CORRECTED SECONDS: " << totalseconds;

		if (totalseconds >= 3600)
		{
			hours =totalseconds / 3600;
			totalseconds -=(hours * 3600);
		}
		else
			hours=0;
		if (totalseconds >= 60)
		{
			minutes =totalseconds / 60;
			totalseconds -= minutes * 60;
		}
		else
			minutes=0;
		seconds = totalseconds;

		if (hours > 23)
		{
			hours = 23;
			minutes=59;
			seconds=59;
		}
		cerr << " HOURS: " << hours << " MINUTES: " << minutes << " SECONDS: " << seconds << endl;

		dud.length = returnLength(hours, minutes, seconds);
		cerr << "cddb:retrieveSearchInfo:FORMATTED TIME: " << dud.length << endl;
		titleInfo.trackData.append(dud);

		location1 = receive.find("TTITLE", location2);
		if (location1 != -1)
		{
			// get the track number
			location1+=6;
			location2 = receive.find("=", location1);
			trackNum=receive.mid(location1, location2-location1).toInt();
			// now get the name of the track
			location1 = location2+1;
			location2 = receive.find("\n", location1);
			// some very long tracks wrap to the next line
			// so if the new tracknum matches the last track we append
			// otherwise it's a new track and we make a new entry
			if (prevNum==trackNum)
			{
				titleInfo.trackData[trackNum].name.append(receive.mid(location1, location2-location1));
			}
			else
			{
				// but it also provides a buffer on error(although I haven't seen any here)
				titleInfo.trackData[trackNum].name = receive.mid(location1, location2 - location1);
				prevNum=trackNum;
			}
			// output result, strip '\n', truncate to 60(the db track size)
			cerr << "cddb::retrieveSearchInfo:TrackNum: " << trackNum << " StringLength: " << titleInfo.trackData[trackNum].name.length();
			cerr << " Name: " << titleInfo.trackData[trackNum].name << endl;
			titleInfo.trackData[trackNum].name.truncate(60);
			if (titleInfo.trackData[trackNum].name[titleInfo.trackData[trackNum].name.length()-1] == '\n')
				titleInfo.trackData[trackNum].name.remove(titleInfo.trackData[trackNum].name.length() - 1, 1);
		}
	}

	return EVERYTHING_OK;
}
/** Queries local CDDB code in KSCD. Based loosely on code found in KSCD. */
int
cddb::queryLocalCDDB()
{
	cerr << "cddb::queryLocalCDDB" << endl;
	// define the variables we will need
	QDir d;
	QStringList pathlist, list;
	QStringList::Iterator it, it2;
	QString rootpath;

	// now lets make sure that cddb/ has subdirs and populate a directory with it
	// taking the first pathlist _should_ be the cddb dir
	d.setPath(serverInfo.local_cddb_path);
	d.setFilter(QDir::Dirs);
	cerr << "cddb::queryLocalCDDB: " << d.path() << endl;
	// now get a list of the directories and we are done with this deal
	rootpath = d.path();
	pathlist = d.entryList();
	//make sure that cddb has more directories in it and not garbage or nothing at all
	if (countSubFSItems(pathlist) < 3)
		return KSCD_MISSING;

	cerr << "cddb::queryLocalCDDB: " << titleInfo.disc_id << endl;

	for (it = pathlist.begin(); it != pathlist.end(); it++)
	{
		d.setPath(rootpath + "/" +(*it));
		cerr << "cddb::queryLocalCDDB: Dir: " << d.path() << " ";
		d.setFilter(QDir::Files);

		if (*it == "." || *it == "..")
		{
			cerr << " we skip." << endl;
			continue;
		}
		list = d.entryList();
		if (!countSubFSItems(list))
			continue;

		for (it2 = list.begin(); it2 != list.end(); it2++)
		{
			if (*it2 == titleInfo.disc_id)
			{
				emit updateGUI(30, i18n("Match Found..."));
				cerr << "cddb::queryLocalCDDB: match: " << *it2 << "\tProcessing..." << endl;

				if (processLocalCDDBFile(rootpath + "/" +(*it) + "/" +(*it2)) == FILE_OPEN_FAIL)
					return FILE_OPEN_FAIL;

				cerr << "cddb::cddbLocalQuery:done" << endl;
				return EVERYTHING_OK;
			}
		}
	}
	cerr << "cddb::cddbLocalQuery:no local match found" << endl;
	return CDDB_NO_MATCH;
}

/**
 * Processes the buffer read from the KSCD CDDB file.
 * @param QString Path To File
 */
int
cddb::processLocalCDDBFile(const QString &ff)
{
	if (ff.isEmpty())
		return FILE_OPEN_FAIL;
	QFile *file = new QFile(validateFilename(ff));
	if (!file)
		exit(1);
	QTextStream *t = new QTextStream(file);
	t->setEncoding(QTextStream::UnicodeUTF8);    
	if (!t)
		exit(1);
	QString read;

	if (!file->open(IO_ReadOnly))
	{
		delete file;
		delete t;
		return FILE_OPEN_FAIL;
	}
	else
	{
		while (!t->eof())
		read += t->readLine() + "\n";
		file->close();
	}
	delete t;
	delete file;

	// process data
	cerr << "cddb::processLocalCDDBFile:Local CDDB File: " << read << endl;
	int seek = 0, seek2 = 0, trackNum=0, prevNum=-1;
	seek = read.find("DTITLE");
	seek = read.find("/", seek);
	seek += 2;
	seek2 = read.find("\n", seek);
	QString name = read.mid(seek, seek2 - seek);
	cerr << "cddb::processLocalCDDBFile:Title Name: " << name << endl;
	titleInfo.title.name = name;

	// now find an equal sign + 1, save till \n, ad infinitum .... adding track names
	for (unsigned int i = 0; i < titleInfo.trackData.count(); i++)
	{
		seek = read.find("TTITLE", seek2);
		if (seek != -1)
		{
			// get the track number
			seek+=6;
			seek2 = read.find("=", seek);
			trackNum=read.mid(seek, seek2-seek).toInt();
			// now get the name of the track
			seek = seek2+1;
			seek2 = read.find("\n", seek);
			// some very long tracks wrap to the next line
			// so if the new tracknum matches the last track we append
			// otherwise it's a new track and we make a new entry
			if (prevNum==trackNum)
			{
				titleInfo.trackData[trackNum].name.append(read.mid(seek, seek2-seek));
			}
			else
			{
				titleInfo.trackData[trackNum].name = read.mid(seek, seek2 - seek);
				prevNum=trackNum;
			}
			// output result, strip '\n', truncate to 60(the db track size)
			cerr << "cddb::processLocalCDDBFile:TrackNum: " << trackNum << " Length: " << titleInfo.trackData[trackNum].name.length();
			cerr << " Name: " << titleInfo.trackData[trackNum].name << endl;
			if (titleInfo.trackData[trackNum].name[titleInfo.trackData[trackNum].name.length()-1] == '\n')
				titleInfo.trackData[trackNum].name.remove(titleInfo.trackData[trackNum].name.length() - 1, 1);
			titleInfo.trackData[trackNum].name.truncate(60);
		}
	}

	return EVERYTHING_OK;
}

/** Says hello to SMTP.
 */
int
cddb::helloSMTP()
{
	QString read = "", write = "";
	int server_return = read_proto(read, serverInfo.put_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return != 220)
		return SMTP_SERVER_BAD_RESPONSE;

	write = "helo ";
	write.append(serverInfo.host);
	write.append("\n");

	if (write_proto(write, serverInfo.put_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	while (server_return==220) {
		server_return=read_proto(read, serverInfo.put_protocol);
	}

	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return != 250)
		return SMTP_EHLO_FAIL;
	return EVERYTHING_OK;
}

/** Sends the email header information.
 */
int
cddb::sendMailHeader()
{
	QString read, write;

	QString frommail;
	settings.setProfile(serverInfo.kde_email_profile);
	cerr << "cddb::sendMailHeader: " << serverInfo.kde_email_profile << endl;
	cerr << "cddb::sendMailHeader: " << settings.currentProfileName() << endl;
	cerr << "cddb::sendMailHeader: " << settings.getSetting(KEMailSettings::ReplyToAddress) << endl;
	frommail = settings.getSetting(KEMailSettings::ReplyToAddress);

	write = "MAIL From:<";
	write.append(frommail);
	write.append(">\n");

	if (write_proto(write, serverInfo.put_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	read="";
	int server_return = read_proto(read, serverInfo.put_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return != 250)
		return SMTP_MAIL_FAIL;

	write = "RCPT To:<";
	write.append(serverInfo.cddb_email);
	write.append(">\n");

	if (write_proto(write, serverInfo.put_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	server_return = read_proto(read, serverInfo.put_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return != 250)
		return SMTP_RCPT_FAIL;

	return EVERYTHING_OK;
}

/** Constructs the Message body from the CDDBtitleInfo.
 * @param QString QString to fill with data
 */
void
cddb::CDDBMessageBody(QString &body)
{
	body = "";
	body.append("Subject: cddb misc ");
	body.append(titleInfo.disc_id);
	body.append("\r\n");
	body.append("From: ");
	body.append(settings.getSetting(KEMailSettings::EmailAddress));
	body.append("\r\n");
	body.append("Reply-To: ");
	body.append(settings.getSetting(KEMailSettings::ReplyToAddress));
	body.append("\r\n");
	body.append("To: ");
	body.append(serverInfo.cddb_email);
	body.append("\r\n\r\n");
	body.append("# xmcd CD database file\r\n# Generated: ");
	body.append(QDateTime::currentDateTime().toString());
	body.append(" by ");
	body.append(PACKAGE);
	body.append("\r\n# \r\n# Track frame offsets:\r\n");
	QValueList < int >::Iterator it;
	for (it = titleInfo.trackSectorBegin.begin(); it != titleInfo.trackSectorBegin.end(); ++it)
	{
		body.append("#       ");
		body.append(QString::number(*it));
		body.append("\r\n");
	}
	body.append("#\r\n# Disc length: ");
	body.append(QString::number((titleInfo.totalLength.toInt()) / 75));
	body.append(" seconds\r\n#\r\n# Revision: 0\r\n# Submitted via: ");
	body.append(PACKAGE);
	body.append(" ");
	body.append(VERSION);
	body.append("\r\n#\r\n");
	body.append("DISCID=");
	body.append(titleInfo.disc_id);
	body.append("\r\nDTITLE=");
	body.append(artist);
	body.append(" / ");
	body.append(titleInfo.title.name);
	body.append("\r\n");
	int i = 0;
	QValueList < trackInfo >::Iterator it2;
	for (it2 = titleInfo.trackData.begin(); it2 != titleInfo.trackData.end(); it2++)
	{
		body.append("TTITLE");
		body.append(QString::number(i));
		body.append("=");
		body.append((*it2).name);
		body.append("\r\n");
		i++;
	}
	body.append("EXTD=\r\n");
	for (int j = 0; j < i; j++)
	{
		body.append("EXTT");
		body.append(QString::number(j));
		body.append("=\r\n");
	}
	body.append("PLAYORDER=\r\n");
	body.append("\r\n.\r\n");
}

/** Sends the body of the message.
 */
int
cddb::sendMailBody()
{
	QString read, write;

	write = "data\n";

	if (write_proto(write, serverInfo.put_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	int server_return = read_proto(read, serverInfo.put_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return != 354)
		return SMTP_BODY_FAIL;

	CDDBMessageBody(write);

	if (write_proto(write, serverInfo.put_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	read = "";
	server_return = read_proto(read, serverInfo.put_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;
	else if (server_return != 250)
		return SMTP_BODY_FAIL;

	return EVERYTHING_OK;
}

/** Says goodbye to SMTP.
 */
int
cddb::goodByeSMTP()
{
	QString read, write;

	write = "quit";

	if (write_proto(write, serverInfo.put_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	return EVERYTHING_OK;
}

// cddb search functions
int cddb::searchQuery()
{
	cerr << "cddb::searchQuery()" << endl;
	cerr << s_text << " " << s_artist << " " << s_title << endl;

	// assemble the search string
	QString write, read;
	write="/freedb_search.php?words=";
	write.append(webify_data(s_text));
	write.append("&allfields=NO");
	if (s_artist)
		write.append("&fields=artist");
	if (s_title)
		write.append("&fields=title");
	write.append("&allcats=YES&grouping=none");

	if (write_proto(write, serverInfo.get_protocol) < 0)
		return SOCKET_WRITE_FAIL;

	int server_return;
	server_return=read_proto(read, serverInfo.get_protocol);
	if (server_return < 0)
		return SOCKET_READ_FAIL;

	int htmlcount=1;
	QString temp;
	while (htmlcount)
	{
		temp="";
		server_return=read_proto(temp, CDDB_PROTO_CDDBP);
		if (server_return < 0)
			return SOCKET_READ_FAIL;
		read.append(temp);
		cerr << "OK" << endl;
		if (temp.find("</body>",0,FALSE) > -1)
			htmlcount--;
	}
	cerr << endl;

	// now time to parse results and store in a qlistview
	emit updateGUI(60,i18n("Parsing Search Results..."));
	read=read.mid(read.find("search results",FALSE));
	read=read.mid(read.find("<tr>",FALSE));
	read.truncate(read.find("</table>",FALSE));
	QStringList lartist, ltitle, lcat, lcdid;
	int f1=0, f2=0;
	QStringList l=QStringList::split("</tr>",read,TRUE);
	l.remove(l.last());
	QStringList::Iterator it;
	for (it=l.begin();it!=l.end();++it)
	{
		f1=(*it).find("cat",FALSE)+4;
		f2=(*it).find("&",f1,FALSE);
		cerr << "CAT: " <<(*it).mid(f1,f2-f1);
		lcat.append((*it).mid(f1,f2-f1));
		f1=f2+4;
		f2=(*it).find("\"",f1,FALSE);
		cerr << " CDID: " <<(*it).mid(f1,f2-f1);
		lcdid.append((*it).mid(f1,f2-f1));
		f1=f2+2;
		f2=(*it).find("/",f1,FALSE)-1;
		cerr << " ARTIST: \"" <<(*it).mid(f1,f2-f1) << "\"";
		lartist.append((*it).mid(f1,f2-f1));
		f1=f2+3;
		f2=(*it).find("</a>",f1,FALSE);
		cerr << " TITLE: \"" <<(*it).mid(f1,f2-f1) << "\"" << endl;
		ltitle.append((*it).mid(f1,f2-f1));
	}

	emit searchReceived(lartist,ltitle,lcat,lcdid);
	return 1;
}

// cddb application functions
/** Pulls the first 3 characters off a QString and converts to a number. Great for checkng typical server return status.
 * @param QString String to parse
 */
int cddb::checkSocketReturnStatus(const QString &serverstring) const
{
	return serverstring.left(3).toInt();
}
/** Wrapper function for read_socket. Reads data from the socket, then reads
 * the error code. Returns if error on reading socket, else returns the protocol
 * error. The data from the socket is written to the passed QString.
 * @param QString Data To Read
 * @param int Protocol
 */
int cddb::read_proto(QString &s, const int &proto)
{
	QString wv;
	if (read_socket(wv) < 0)
		return -1;
	cerr << "cddb::read_proto: " << wv;

	switch(proto)
	{
		case CDDB_PROTO_CDDBP:
		case CDDB_PROTO_EMAIL:
			break;
		case CDDB_PROTO_HTTP:
		case CDDB_PROTO_PROXY:
		case CDDB_PROTO_SEARCH:
		case CDDB_PROTO_SEARCHPROXY:
			while (wv != "\r\n")
			{
				wv="";
				if (read_socket(wv) <= 0)
					return -1;
				cerr << "cddb::read_proto: " << wv;
			}
			wv="";
			if (read_socket(wv) <= 0)
				return -1;
			cerr << "cddb::read_proto: " << wv;
			break;
		default:
			return -1;
	}
	s=wv;
	return checkSocketReturnStatus(s);
}

/** Wrapper function for write_socket. Sends the cddb message in the correct form
 * depending on the protocol(cddbp, http, proxy).
 * Returns write_socket result.
 * @param QString Data To Write
 * @param int Protocol
 */
int cddb::write_proto(const QString &s, const int &proto)
{
	if (s.isEmpty())
		return -1;
	QString wv;
	switch(proto)
	{
		case CDDB_PROTO_CDDBP:
		case CDDB_PROTO_EMAIL:
			wv=s;
			break;
		case CDDB_PROTO_SEARCH:
			wv="GET "+s+" HTTP/1.0\r\n\r\n";
			break;
		case CDDB_PROTO_SEARCHPROXY:
			wv="GET http://"+serverInfo.http_server+":"+
				serverInfo.http_port+s+" HTTP/1.0\r\n\r\n";
			break;
		case CDDB_PROTO_HTTP:
			wv="GET "+serverInfo.http_cgi+"?cmd="+
				webify_data(s)+"&hello="+getenv("USER")+
				"+"+serverInfo.host+"+KmusicdB+"+VERSION+
				"&proto=4"+" HTTP/1.0\r\n\r\n";
			break;
		case CDDB_PROTO_PROXY:
			wv="GET http://"+serverInfo.http_server+
				":"+serverInfo.http_port+serverInfo.http_cgi+"?cmd="+
				webify_data(s)+"&hello="+getenv("USER")+
				"+"+serverInfo.host+"+KmusicdB+"+VERSION+
				"&proto=4"+" HTTP/1.0\r\n\r\n";
			break;
		default:
			break;
	};
	cerr << "cddb::write_proto: " << wv << endl;
	if (write_socket(wv) < 0)
		return -1;
	return EVERYTHING_OK;
}
/** Rewrites CDDB message to be compatible with HTTP servers.
 * Returns HTTP compatible message.
 * @param QString Data To Translate
 */
QString cddb::webify_data(const QString &sstr)
{
	QString t = sstr;
	QString badchars = "?=+%&";
	int badcharslen = badchars.length();
	QString replacement;
	for (unsigned int i = 0; i < t.length(); i++)
	{
		if (t[i].isSpace())
		{
			t[i]='+';
			continue;
		}
		for (int j = 0; j < badcharslen; j++)
		{
			if (t[i]==badchars[j])
			{
				replacement.sprintf("%%%02X",t[i].latin1());
				t.remove(i,1);
				t.insert(i,replacement);
				i+=3;
			}
		}
	}
	return t;
}




// socket related functions
/** Does the host lookup and makes the connection to the server.
 * QString Connecting Host
 * QString Connecting Port
 */
int
cddb::connectToServer(const QString &host, const QString &port)
{
	if (host.isEmpty() || port.isEmpty())
	{
		emit error(SOCKET_HOST_NOT_FOUND);
		return -1;
	}
	cerr << "cddb::connectToServer: " << host << ":" << port << endl;
	sockfd = 0;
	errno = 0;

	bzero(&socketaddr, sizeof(socketaddr));
	socketaddr.sin_family = AF_INET;
	socketaddr.sin_port = htons(port.toInt());

	kapp->processEvents();
	struct hostent *dnslookup = gethostbyname(host.latin1());
	if (dnslookup == NULL)
		goto handle_error;
	memcpy(&socketaddr.sin_addr,(struct in_addr *)(dnslookup->h_addr), sizeof(struct in_addr));
//	cerr << "Host Found: " << dnslookup->h_addr << endl;
	emit updateGUI(35, i18n("Host found..."));

	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0)
	{
		emit error(SOCKET_CREATE_FAILED);
		return -1;
	}

	if (::connect(sockfd,(struct sockaddr *) &socketaddr, sizeof(socketaddr)) == 0)
		emit updateGUI(37, i18n("Connected to host..."));
	else
	{
		cerr << "cddb::connectToServer:Could not connect to host." << endl;
		goto handle_error;
	}
	cerr << "cddb::connectToServer:Connected to host." << endl;

	emit sigconnected();
	return EVERYTHING_OK;

handle_error:
	if (errno == EINTR)
		emit error(SOCKET_TIMEOUT);
	else
		emit error(SOCKET_HOST_NOT_FOUND);
	shutdown(sockfd, SHUT_RDWR);
	return -1;

}

/** Reads the socket. Data is appended to QString.
 * @param QString Append Data To
 */
int
cddb::read_socket(QString &data)
{
	// setup out descriptor;
	fd_set rset;
	FD_ZERO(&rset);
	FD_SET(sockfd, &rset);

	// setup our timeval
	struct timeval tv;
	tv.tv_sec = DEFAULT_SOCKET_TIMEOUT;
	tv.tv_usec = 0;

	// we will need a few variables, and better set errno to 0 just in case
	int nread = 0, rselect = 0;
	QString receive = "";
	char t;
	errno = 0;

	// loop the select and read until we get a whole line of text off the socket
	do
	{
		// a selecting we will go, a selecting we will go
		rselect = select(sockfd + 1, &rset, NULL, NULL, &tv);
		if (rselect < 0)
		{
			cerr << "cddb::read_socket: Select Failed." << endl;
			goto handle_error;
		}
		else if (rselect == 0)
		{
			cerr << "cddb::read_socket: Select TimedOut" << endl;
			goto handle_error;
		}

		// read a byte.
		nread = read(sockfd, &t, 1);
		if (nread == 1)		// 1byte read
		{
			receive.append(t);
			if (t == '\n')
				break;
		}
		else if (nread < 0)	// error.. handle it
		{
			cerr << "cddb::read_socket: nread<0" << endl;
			goto handle_error;
		}
		else			// no bytes read, emtpy socket
		{
			cerr << "cddb::read_socket: 0,>1" << endl;
			goto handle_error;
		}
	}
	while (nread >= 0);
	data.append(receive);

	return EVERYTHING_OK;

handle_error:
	emit error(SOCKET_READ_FAIL);
	return -1;
}

/** Writes to the socket.
 * @param QString Data to write.
 */
int
cddb::write_socket(const QString &data)
{
	if (data.isEmpty())
	{
		emit error(SOCKET_WRITE_FAIL);
		return -1;
	}
	// setup out descriptor;
	fd_set rset;
	FD_ZERO(&rset);
	FD_SET(sockfd, &rset);

	// setup our timeval
	struct timeval tv;
	tv.tv_sec = DEFAULT_SOCKET_TIMEOUT;
	tv.tv_usec = 0;

	int nread = 0, rselect = 0;
	rselect = select(sockfd + 1, NULL, &rset, NULL, &tv);
	if (rselect < 0)
	{
		cerr << "cddb::write_socket: Selected Failure." << endl;
		goto handle_error;
	}
	else if (rselect == 0)
	{
		cerr << "cddb::write_socket: Select Timed Out" << endl;
		emit error(SOCKET_TIMEOUT);
		return -1;
	}
	if ((nread = write(sockfd, data.latin1(), data.length())) <= 0)
		goto handle_error;

	return EVERYTHING_OK;

handle_error:
	emit error(SOCKET_WRITE_FAIL);
	return -1;
}

#include "cddb.moc"
