/***************************************************************************
                          cddb.h  -  CDDB and cdparanoia classes
                             -------------------
    begin                : Sat Aug 18 20:20:00 /etc/localtime 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDDB_H
#define CDDB_H

#include "include.h"
#include "kmusicdbvalidator.h"
#include "kmusicdbfile.h"

/**
 *The cdparanoia class is designed to retrieve CDDA track information from the CD and return it in the
 * CDDBtitleInfo struct.
 *
 *<pre>
 *cdparanoia *cdp=new cdparanoia(path);
 *if(cdp->cdCheckInDrive(device))
 *	cdp->get(CDDBtitleInfo*);
 *</pre>
 * @short cdparanoia processing class
 * @author Will DeRousse
 */

class cdparanoia : public QObject {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QString Path to cdparanoia if it is not in path
		 * @param CDDBtitleInfo Pointer To Struct
		 */
		cdparanoia(const QString &, struct CDDBtitleInfo &);
		/**
		 * Destructor
		 */
		~cdparanoia();
		/** Get cdparanoia information from CD-ROM drive.
		 */
		void get();
	signals:
		/**
		 * Done processing events.
		 */
		void cdpDone(CDDBtitleInfo);
		/** Finished with whole process or error.
		 */
		void cdpFinish();
		/** Error signal.
		 * @param int Error type defined in error.h
		 */
		void error(int);
		/** Used to update the status bar.
		 * @param int Value Of Statusbar
		 * @param QString Text to set Statusbar with
		 */
		void updateGUI(int,QString);
	private slots:
		/**
		 * Called when the cdparanoia timer timesout. This function kills the current process, emits error() and returns.
		 */
		void cdpTimeout();
		/** Processes the output of cdparanoia after it is done executing.
		 * @param KProcess Process
		 */
		void procExit(KProcess *);
		/** Just appends the buffer to cdparaout.
		 * @param KProcess The Process
		 * @param char Pointer to the data returned
		 * @param int Amount of data available
		 */
		void procOutput(KProcess *, char *, int);
	private:
		/** Checks to find out where cdparanoia is located.
		 */
		bool checkPath();
		/**
		 * Process the output of cdparanoia.
		 */
		int processEvents();
		/**
		 * Does part of the summation for calculating disc_id.
		 *
		 * Returns sum.
		 */
		int cddbSum();

		QString path, cdparaout;
		KProcess p;
		QTimer *cdptimer;
		CDDBtitleInfo titleInfo;
};


/**
 *The cdparanoia class is designed to retrieve or submit CDDB information. It is fashioned similarly
 * to an ioslave and could easily be converted. However, it is NOT an ioslave it just contains convenient
 * get and put functions.
 *
 *<pre>
 *cddb *cddb2=new cddb(parent, CDDBtitleInfo &);
 *connect(cddb signals);
 *cddb2->get(...);
 *cddb2->put(...);
 *</pre>
 * @short Retrieve and submit CDDB information.
 * @author Will DeRousse
 */

class cddb : public QObject {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent Widget
		 * @param CDDBtitleInfo CDDBtitleInfo to set default info and pass track information
		 * @param CDDBserverInfo CDDBserverInfo holds server/protocol information
		 */
		cddb(QWidget *, struct CDDBtitleInfo, struct CDDBserverInfo *);
		/**
		 * Destructor
		 */
		~cddb();
		/** get function for retrieving CDDB info. Must have cdparanoia information first.
		 */
		void get();
		/** For submitting to CDDB.
		 * @param QString Artist Name
		 */
		void put(const QString &);
		/** For searching CDDB.
		 * @param QString Search String
		 * @param bool Artist Enabled Or Disabled
		 * @param bool Title Enabled Or Disabled
		 */
		void search(const QString &, const bool &, const bool &);
		/** For retrieving titles if you already know the genre and discid.
		 * @param QString CDDB Category
		 * @param QString CDDB Disc ID
		 */
		void simpleGet(const QString &, const QString &);
	signals:
		// generic signals
		/** Error Processing Signal.
		 * @param int Error Type
		 */
		void error(int);
		/** Emitted during CDDB_STATE_SHUTDOWN either when an error occurs or when a CDDB connection normally terminates.
		 */
		void cddbFinished();
		/** Emitted when the connection to the server is complete.
		 */
		void sigconnected();

		// signals for cddb
		/** When search is complete and has been decoded to a QListView.
		 * @param QStringList Artist
		 * @param QStringList Title
		 * @param QStringList CDDB Category
		 * @param QStringList CDDB Disc ID
		 */
		void searchReceived(QStringList, QStringList, QStringList, QStringList);
		/** When retrievel has been completed, this signal is emitted.
		 * @param CDDBtitleInfo Struct With CDDB Info
		 */
		void titleReceived(CDDBtitleInfo);
		/** For updating the status bar.
		 * @param int Progressbar value
		 * @param QString Text to set statusbar
		 */
		void updateGUI(int,QString);

		// signals for smtp
		/** Emitted when the email submission is complete.
		 * @param CDDBtitleInfo Struct with CDDB Info
		 */
		void submitSuccess(CDDBtitleInfo);
	private slots:
		// generic slots

		// cddb slots
		/** Retrieves the CD information from CDDB. Main wrapper function.
		 */
		void cddb_spincycle();

		// socket slots

	private:
		// generic functions

		// cddb functions
		/**
		  Queries local CDDB code in KSCD. Based loosely on code found in KSCD.
		 */
		int queryLocalCDDB();

		/** Says hello to CDDB.
		 */
		int helloCDDB();
		/** Sends the query to CDDB.
		 */
		int queryForCD();
		/** The work horse, retrieves and processes the title information on query.
		 */
		int retrieveCDInfo();
		/** The work horse, retrieves and processes the title information on search.
		 */
		int retrieveSearchInfo();
		/** Says goodbye to CDDB.
		 */
		int goodByeCDDB();

		// search fns
		/** Searches the CDDB database for a title based on text query.
		 */
		int searchQuery();
		// smtp functions
		/** Constructs the Message body from the CDDBtitleInfo.
		 * @param QString QString to fill with data
		 */
		void CDDBMessageBody(QString &);
		/** Says hello to SMTP.
		 */
		int helloSMTP();
		/** Sends the email header information.
		 */
		int sendMailHeader();
		/** Sends the body of the message.
		 */
		int sendMailBody();
		/** Says goodbye to SMTP.
		 */
		int goodByeSMTP();
		/**
		 * Processes the buffer read from the KSCD CDDB file.
 		 * @param QString Path To File
		 */
		int processLocalCDDBFile(const QString &);

		// cddb protocol functions
		/** Wrapper function for write_socket. Sends the cddb message in the correct form
		 * depending on the protocol (cddbp, http, proxy).
		 * Returns write_socket result.
		 * @param QString Data To Write
		 * @param int Protocol
		 */
		int write_proto(const QString &, const int &);
		/** Wrapper function for read_socket. Reads data from the socket, then reads
		 * the error code. Returns if error on reading socket, else returns the protocol
		 * error. The data from the socket is written to the passed QString.
		 * @param QString Data To Read
		 * @param int Protocol
		 */
		int read_proto(QString &, const int &);
		/** Rewrites CDDB message to be compatible with HTTP servers.
		 * Returns HTTP compatible message.
		 * @param QString Data To Translate
		 */
		QString webify_data(const QString &);
		/** Pulls the first 3 characters off a QString and converts to a number. Great for checkng typical server return status.
		 * @param QString String to parse
		 */
		int checkSocketReturnStatus(const QString &) const;
		// socket level functions
		/** Does the host lookup and makes the connection to the server.
		 * QString Connecting Host
		 * QString Connecting Port
		 */
		int connectToServer(const QString &, const QString &);
		/** Reads the socket. Data is appended to QString.
		 * @param QString Append Data To
		 */
		int read_socket(QString &);
		/** Writes to the socket.
		 * @param QString Data To Write
		 */
		int write_socket(const QString &);

		// generic variables
		CDDBtitleInfo titleInfo;
		CDDBserverInfo serverInfo;
		QString chost, cport;
		int state, preservestate;

		// cddb variables

		// search variables
		QString s_text;
		bool s_artist, s_title;

		// smtp variables
		QString submitdata, artist;
		KEMailSettings settings;

		// socket variables
		int sockfd;
		struct sockaddr_in socketaddr;
};


#endif

