/***************************************************************************
                          musicqlistview.h  -  description
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MUSICQLISTVIEW_H
#define MUSICQLISTVIEW_H

#include "include.h"
#include "kmusicdbvalidator.h"

/**This is the CDDBtitleInfo drag and drop class. It is used for dragging and dropping titles.
 *
 * @author Will DeRousse
 * @short CDDBtitleInfo Drag and Drop
 */

class CDDBtitleInfoDrag : public QDragObject
{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param CDDBtitleInfo Struct containing drag data
		 * @param char Name
		 */
		CDDBtitleInfoDrag(QWidget *, const CDDBtitleInfo &, const char *name = 0);
		/**
		 * Used to determine if this class can decode the QMimeSource or not.
		 * @param QMimeSource Dropped Data
		 */
		static bool canDecode(QMimeSource *);
		/**
		 * Decodes the dropped data.
		 * @param QDropEvent Event information regarding drop
		 * @param CDDBtitleInfo Struct to be filled
		 */
		static bool decode(QDropEvent *, CDDBtitleInfo &);
		/**
		 * Utility function for encoding a CDDBtitleInfo struct.
		 * @param CDDBtitleInfo Struct to be encoded into class
		 */
		void encode(const CDDBtitleInfo &);
		/**
		 * Returns QByteArray encoded data.
		 * @param char MIME Type
		 */
		QByteArray encodedData(const char *) const;
		/**
		 * Returns supported formats.
		 * @param int Format Number
		 */
		const char *format(int) const;
	private:
		CDDBtitleInfo info;

};

/**This is the trackInfo drag and drop class. It is used for dragging and dropping tracks.
 *
 * @author Will DeRousse
 * @short trackInfo Drag and Drop
 */

class trackInfoDrag : public QDragObject
{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param trackInfo Struct containing drag data
		 * @param name Name
		 */
		trackInfoDrag(QWidget *, const trackInfo &, const char *name = 0);
		/**
		 * Used to determine if this class can decode the QMimeSource or not.
		 * @param QMimeSource Dropped Data
		 */
		static bool canDecode(QMimeSource *);
		/**
		 * Decodes the dropped data.
		 * @param QDropEvent Event information regarding drop
		 * @param trackInfo Struct to be filled
		 */
		static bool decode(QDropEvent *, trackInfo &);
		/**
		 * Utility function for encoding a CDDBtitleInfo struct.
		 * @param trackInfo Struct to be encoded into class
		 */
		void encode(const trackInfo &);
		/**
		 * Returns QByteArray encoded data.
		 * @param char MIME Type
		 */
		QByteArray encodedData(const char *) const;
		/**
		 * Returns supported formats.
		 * @param int Format Number
		 */
		const char* format(int) const;
	private:
		trackInfo info;

};

/**
 * This is the title QListView for musicArtistTitle. It has special signals and attributes
 * to handle graying and clicking. Also implemented is QTextDrag for dragging and dropping.
 * It is used for dragging and dropping in the category dialogs.
 *
 * @author Will DeRousse
 * @short QListView with special attributes (and text drag and drop)..
 */

class musicQListView : public QListView  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param int Drag 'N Drop Type: 0 none, 1 QString, 2 trackInfo, 3 CDDBtitleInfo
		 */
		musicQListView(QWidget *, const int &);
		/**
		 * Destructor
		 */
		~musicQListView();
		/**
		 * Returns width of QListView.
		 */
		int specialWidth();
		/**
		 * Handles whether or not the data can be decoded or not.
		 * This class decodes text.
		 * @param QDragEnterEvent Event pertaining to drag enter
		 */
		void dragEnterEvent(QDragEnterEvent *);
		/**
		 * Emits dropped(QString) after decode.
		 * @param QDropEvent Event pertaining to dropping
		 */
		void dropEvent(QDropEvent *);
		/**
		 * Handles whether or not the data should be copied for dragging and dropping.
		 * Encodes the category name.
		 * @param QMouseEvent Mouse Movement Event Data
		 */
		void contentsMouseMoveEvent(QMouseEvent *);
		/**
		 * Allows an item to be "ensureItemVisible" by specifying CDID rather than pointer.
		 */
		void ensureItemVisibleByCDID(const QString &);
	signals:
		/** Emitted when focus is gained.
		 * @param bool TRUE focus gained, FALSE focus lost
		 */
		void imFocused(bool);
		/**
		 * Emitted when text has been dropped on the QListView.
		 * @param QString Dropped Text
		 */
		void dropped(QString);
		/**
		 * Emitted when a trackInfo struct has been dropped.
		 * Used for external access to database.
		 * @param trackInfo Dropped Struct
		 */
		void dropped(trackInfo);
		/**
		 * Emitted when a CDDBtitleInfo struct has been dropped.
		 * Used for external access to database.
		 * @param CDDBtitleInfo Dropped struct
		 */
		void dropped(CDDBtitleInfo);
		/**
		 * Emitted when a QListViewItem is dragged.
		 * Used for external access to database.
		 * @param QListView Pointer to QListView
		 */
		void dragged(QListView *);
	private slots:
		/**
		 * Handles mouse and keyboard selection events for emitted imFocused.
		 */
		void handleKeyEvents();
	private:
		int dndtype;
};

#endif

